@@ -19,7 +19,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 #include "gtktooltip.h"
 #include "gtkintl.h"
 #include "gtkwindow.h"
@@ -71,6 +71,7 @@ struct _GtkTooltip
   guint browse_mode_enabled : 1;
   guint keyboard_mode_enabled : 1;
   guint tip_area_set : 1;
+  guint custom_was_reset : 1;
 };
 
 struct _GtkTooltipClass
@@ -92,6 +93,8 @@ static void       gtk_tooltip_window_hide          (GtkWidget       *widget,
 static void       gtk_tooltip_display_closed       (GdkDisplay      *display,
 						    gboolean         was_error,
 						    GtkTooltip      *tooltip);
+static void       gtk_tooltip_set_last_window      (GtkTooltip      *tooltip,
+						    GdkWindow       *window);
 
 
 G_DEFINE_TYPE (GtkTooltip, gtk_tooltip, G_TYPE_OBJECT);
@@ -141,9 +144,9 @@ gtk_tooltip_init (GtkTooltip *tooltip)
   gtk_container_add (GTK_CONTAINER (tooltip->window), tooltip->alignment);
   gtk_widget_show (tooltip->alignment);
 
-  g_signal_connect_swapped (tooltip->window, "style_set",
+  g_signal_connect_swapped (tooltip->window, "style-set",
 			    G_CALLBACK (gtk_tooltip_window_style_set), tooltip);
-  g_signal_connect_swapped (tooltip->window, "expose_event",
+  g_signal_connect_swapped (tooltip->window, "expose-event",
 			    G_CALLBACK (gtk_tooltip_paint_window), tooltip);
 
   tooltip->box = gtk_hbox_new (FALSE, tooltip->window->style->xthickness);
@@ -179,6 +182,8 @@ gtk_tooltip_finalize (GObject *object)
       tooltip->browse_mode_timeout_id = 0;
     }
 
+  gtk_tooltip_set_last_window (tooltip, NULL);
+
   if (tooltip->window)
     {
       GdkDisplay *display;
@@ -274,7 +279,7 @@ gtk_tooltip_set_icon (GtkTooltip *tooltip,
 /**
  * gtk_tooltip_set_icon_from_stock:
  * @tooltip: a #GtkTooltip
- * @stock_id: a stock icon name, or %NULL
+ * @stock_id: a stock id, or %NULL
  * @size: a stock icon size
  *
  * Sets the icon of the tooltip (which is in front of the text) to be
@@ -298,6 +303,33 @@ gtk_tooltip_set_icon_from_stock (GtkTooltip  *tooltip,
     gtk_widget_hide (tooltip->image);
 }
 
+/**
+ * gtk_tooltip_set_icon_from_icon_name:
+ * @tooltip: a #GtkTooltip
+ * @icon_name: an icon name, or %NULL
+ * @size: a stock icon size
+ *
+ * Sets the icon of the tooltip (which is in front of the text) to be
+ * the icon indicated by @icon_name with the size indicated
+ * by @size.  If @icon_name is %NULL, the image will be hidden.
+ *
+ * Since: 2.14
+ */
+void
+gtk_tooltip_set_icon_from_icon_name(GtkTooltip  *tooltip,
+				    const gchar *icon_name,
+				    GtkIconSize  size)
+{
+  g_return_if_fail (GTK_IS_TOOLTIP (tooltip));
+
+  gtk_image_set_from_icon_name (GTK_IMAGE (tooltip->image), icon_name, size);
+
+  if (icon_name)
+    gtk_widget_show (tooltip->image);
+  else
+    gtk_widget_hide (tooltip->image);
+}
+
 /**
  * gtk_tooltip_set_custom:
  * @tooltip: a #GtkTooltip
@@ -318,6 +350,15 @@ gtk_tooltip_set_custom (GtkTooltip *tooltip,
   if (custom_widget)
     g_return_if_fail (GTK_IS_WIDGET (custom_widget));
 
+  /* The custom widget has been updated from the query-tooltip
+   * callback, so we do not want to reset the custom widget later on.
+   */
+  tooltip->custom_was_reset = TRUE;
+
+  /* No need to do anything if the custom widget stays the same */
+  if (tooltip->custom_widget == custom_widget)
+    return;
+
   if (tooltip->custom_widget)
     {
       GtkWidget *custom = tooltip->custom_widget;
@@ -356,8 +397,8 @@ gtk_tooltip_set_custom (GtkTooltip *tooltip,
  * Since: 2.12
  */
 void
-gtk_tooltip_set_tip_area (GtkTooltip   *tooltip,
-			  GdkRectangle *rect)
+gtk_tooltip_set_tip_area (GtkTooltip         *tooltip,
+			  const GdkRectangle *rect)
 {
   g_return_if_fail (GTK_IS_TOOLTIP (tooltip));
 
@@ -399,6 +440,10 @@ gtk_tooltip_trigger_tooltip_query (GdkDisplay *display)
   event.motion.y = y;
   event.motion.is_hint = FALSE;
 
+  gdk_window_get_origin (window, &x, &y);
+  event.motion.x_root = event.motion.x + x;
+  event.motion.y_root = event.motion.y + y;
+
   _gtk_tooltip_handle_event (&event);
 }
 
@@ -409,8 +454,12 @@ gtk_tooltip_reset (GtkTooltip *tooltip)
 {
   gtk_tooltip_set_markup (tooltip, NULL);
   gtk_tooltip_set_icon (tooltip, NULL);
-  gtk_tooltip_set_custom (tooltip, NULL);
   gtk_tooltip_set_tip_area (tooltip, NULL);
+
+  /* See if the custom widget is again set from the query-tooltip
+   * callback.
+   */
+  tooltip->custom_was_reset = FALSE;
 }
 
 static void
@@ -473,6 +522,9 @@ child_location_foreach (GtkWidget *child,
   if (!GTK_WIDGET_DRAWABLE (child))
     return;
 
+  x = 0;
+  y = 0;
+
   /* (child_loc->x, child_loc->y) are relative to
    * child_loc->container's allocation.
    */
@@ -704,7 +756,22 @@ gtk_tooltip_display_closed (GdkDisplay *display,
 			    gboolean    was_error,
 			    GtkTooltip *tooltip)
 {
-  g_object_set (display, "gdk-display-current-tooltip", NULL);
+  g_object_set_data (G_OBJECT (display), "gdk-display-current-tooltip", NULL);
+}
+
+static void
+gtk_tooltip_set_last_window (GtkTooltip *tooltip,
+			     GdkWindow  *window)
+{
+  if (tooltip->last_window)
+    g_object_remove_weak_pointer (G_OBJECT (tooltip->last_window),
+				  (gpointer *) &tooltip->last_window);
+
+  tooltip->last_window = window;
+
+  if (window)
+    g_object_add_weak_pointer (G_OBJECT (tooltip->last_window),
+			       (gpointer *) &tooltip->last_window);
 }
 
 static gboolean
@@ -746,6 +813,12 @@ gtk_tooltip_run_requery (GtkWidget  **widget,
     }
   while (*widget);
 
+  /* If the custom widget was not reset in the query-tooltip
+   * callback, we clear it here.
+   */
+  if (!tooltip->custom_was_reset)
+    gtk_tooltip_set_custom (tooltip, NULL);
+
   return return_value;
 }
 
@@ -845,6 +918,7 @@ gtk_tooltip_show_tooltip (GdkDisplay *display)
 
   if (tooltip->keyboard_mode_enabled)
     {
+      x = y = -1;
       pointer_widget = tooltip_widget = tooltip->keyboard_widget;
     }
   else
@@ -1090,6 +1164,9 @@ _gtk_tooltip_toggle_keyboard_mode (GtkWidget *widget)
       g_object_set_data_full (G_OBJECT (display),
 			      "gdk-display-current-tooltip",
 			      tooltip, g_object_unref);
+      g_signal_connect (display, "closed",
+			G_CALLBACK (gtk_tooltip_display_closed),
+			tooltip);
     }
 
   tooltip->keyboard_mode_enabled ^= 1;
@@ -1132,24 +1209,35 @@ _gtk_tooltip_hide (GtkWidget *widget)
     gtk_tooltip_hide_tooltip (tooltip);
 }
 
+static gboolean
+tooltips_enabled (GdkWindow *window)
+{
+  gboolean enabled;
+  gboolean touchscreen;
+  GdkScreen *screen;
+  GtkSettings *settings;
+
+  screen = gdk_drawable_get_screen (window);
+  settings = gtk_settings_get_for_screen (screen);
+
+  g_object_get (settings,
+		"gtk-touchscreen-mode", &touchscreen,
+		"gtk-enable-tooltips", &enabled,
+		NULL);
+
+  return (!touchscreen && enabled);
+}
+
 void
 _gtk_tooltip_handle_event (GdkEvent *event)
 {
   gint x, y;
   gboolean return_value = FALSE;
-  gboolean touchscreen;
   GtkWidget *has_tooltip_widget = NULL;
-  GdkScreen *screen;
   GdkDisplay *display;
   GtkTooltip *current_tooltip;
-  GtkSettings *settings;
 
-  /* Disable tooltips in touchscreen mode */
-  screen = gdk_drawable_get_screen (event->any.window);
-  settings = gtk_settings_get_for_screen (screen);
-  g_object_get (settings, "gtk-touchscreen-mode", &touchscreen, NULL);
-
-  if (touchscreen)
+  if (!tooltips_enabled (event->any.window))
     return;
 
   /* Returns coordinates relative to has_tooltip_widget's allocation. */
@@ -1160,7 +1248,7 @@ _gtk_tooltip_handle_event (GdkEvent *event)
 
   if (current_tooltip)
     {
-      current_tooltip->last_window = event->any.window;
+      gtk_tooltip_set_last_window (current_tooltip, event->any.window);
       gdk_event_get_root_coords (event,
 				&current_tooltip->last_x,
 				&current_tooltip->last_y);
@@ -1264,8 +1352,11 @@ _gtk_tooltip_handle_event (GdkEvent *event)
 	    g_object_set_data_full (G_OBJECT (display),
 				    "gdk-display-current-tooltip",
 				    current_tooltip, g_object_unref);
+	    g_signal_connect (display, "closed",
+			      G_CALLBACK (gtk_tooltip_display_closed),
+			      current_tooltip);
 
-	    current_tooltip->last_window = event->any.window;
+	    gtk_tooltip_set_last_window (current_tooltip, event->any.window);
 	    gdk_event_get_root_coords (event,
 				       &current_tooltip->last_x,
 				       &current_tooltip->last_y);
